@@ -20,6 +20,10 @@
  * Author: Shaun McCance <shaunm@gnome.org>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdarg.h>
 
 #include <gtk/gtk.h>
@@ -36,6 +40,8 @@ struct _YelpSettingsPriv {
     gchar        *icons[YELP_SETTINGS_NUM_ICONS];
     gint          icon_size;
 
+    GHashTable   *pixbufs;
+
     GtkSettings  *gtk_settings;
     GtkIconTheme *gtk_icon_theme;
 
@@ -48,6 +54,8 @@ struct _YelpSettingsPriv {
     gboolean      show_text_cursor;
 
     gboolean      editor_mode;
+
+    GHashTable   *bookmarks;
 };
 
 enum {
@@ -171,7 +179,7 @@ yelp_settings_class_init (YelpSettingsClass *klass)
                                      PROP_SHOW_TEXT_CURSOR,
                                      g_param_spec_boolean ("show-text-cursor",
                                                            _("Show Text Cursor"),
-                                                           _("Show the text cursor or caret for accessibile navigation"),
+                                                           _("Show the text cursor or caret for accessible navigation"),
                                                            FALSE,
                                                            G_PARAM_READWRITE | G_PARAM_STATIC_NAME |
                                                            G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
@@ -221,12 +229,18 @@ yelp_settings_init (YelpSettings *settings)
     settings->priv->mutex = g_mutex_new ();
     settings->priv->icon_size = 24;
 
+    settings->priv->pixbufs = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                     g_free, g_object_unref);
+
     for (i = 0; i < YELP_SETTINGS_NUM_ICONS; i++)
 	settings->priv->icons[i] = NULL;
     for (i = 0; i < YELP_SETTINGS_NUM_FONTS; i++) {
 	settings->priv->setfonts[i] = NULL;
 	settings->priv->fonts[i] = NULL;
     }
+
+    settings->priv->bookmarks = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                       g_free, NULL);
 }
 
 static void
@@ -234,6 +248,11 @@ yelp_settings_dispose (GObject *object)
 {
     YelpSettings *settings = YELP_SETTINGS (object);
 
+    if (settings->priv->pixbufs != NULL) {
+        g_hash_table_destroy (settings->priv->pixbufs);
+        settings->priv->pixbufs;
+    }
+
     G_OBJECT_CLASS (yelp_settings_parent_class)->dispose (object);
 }
 
@@ -671,6 +690,20 @@ yelp_settings_get_icon_param (YelpSettingsIcon icon)
 
 /******************************************************************************/
 
+GdkPixbuf *
+yelp_settings_get_icon_pixbuf (YelpSettings *settings,
+                               const gchar  *icon_name)
+{
+    GdkPixbuf *pixbuf;
+
+    pixbuf = g_hash_table_lookup (settings->priv->pixbufs, icon_name);
+    if (pixbuf != NULL)
+        return g_object_ref (pixbuf);
+
+    return gtk_icon_theme_load_icon (settings->priv->gtk_icon_theme,
+                                     icon_name, 16, 0, NULL);
+}
+
 gboolean
 yelp_settings_get_show_text_cursor (YelpSettings *settings)
 {
@@ -911,9 +944,12 @@ icon_theme_changed (GtkIconTheme *theme,
 {
     GtkIconInfo *info;
     gint i;
+    GdkPixbuf *pixbuf;
+    GdkRGBA color, base;
 
     g_mutex_lock (settings->priv->mutex);
 
+    /* Set Yelp's note icons */
     for (i = 0; i < YELP_SETTINGS_NUM_ICONS; i++) {
 	if (settings->priv->icons[i] != NULL)
 	    g_free (settings->priv->icons[i]);
@@ -931,6 +967,94 @@ icon_theme_changed (GtkIconTheme *theme,
 	}
     }
 
+    /* Set pixbufs for named icons */
+    gdk_rgba_parse (&color, settings->priv->colors[YELP_SETTINGS_COLOR_TEXT_LIGHT]);
+    gdk_rgba_parse (&base, settings->priv->colors[YELP_SETTINGS_COLOR_BASE]);
+
+    info = gtk_icon_theme_lookup_icon (theme, "edit-clear-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("edit-clear"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "edit-find-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("system-search"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-bookmark-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info,
+                                          &color, &base,
+                                          NULL, NULL, NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-bookmark-add"), pixbuf);
+    pixbuf = gtk_icon_info_load_symbolic (info,
+                                          &color, &color,
+                                          NULL, NULL, NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-bookmark-remove"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-task-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-task"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-tip-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-tip"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-video-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-video"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-ui-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-ui"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-problem-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-problem"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-search-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page-search"), pixbuf);
+    gtk_icon_info_free (info);
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-page-symbolic", 16,
+                                       GTK_ICON_LOOKUP_FORCE_SVG);
+    pixbuf = gtk_icon_info_load_symbolic (info, &color,
+                                          NULL, NULL, NULL,
+                                          NULL, NULL);
+    g_hash_table_replace (settings->priv->pixbufs, g_strdup ("yelp-page"), pixbuf);
+    gtk_icon_info_free (info);
+
     g_mutex_unlock (settings->priv->mutex);
 
     g_signal_emit (settings, settings_signals[ICONS_CHANGED], 0);
@@ -941,17 +1065,19 @@ yelp_settings_cmp_icons (const gchar *icon1,
                          const gchar *icon2)
 {
     static const gchar *icons[] = {
-        "yelp-page-task",
+        "yelp-page-search",
         "yelp-page-video",
+        "yelp-page-task",
         "yelp-page-tip",
+        "yelp-page-problem",
         "yelp-page-ui",
-        "help-contents",
+        "yelp-page",
         NULL
     };
     gint i;
     for (i = 0; icons[i] != NULL; i++) {
-        gboolean eq1 = icon1 ? g_str_equal (icon1, icons[i]) : FALSE;
-        gboolean eq2 = icon2 ? g_str_equal (icon2, icons[i]) : FALSE;
+        gboolean eq1 = icon1 ? g_str_has_prefix (icon1, icons[i]) : FALSE;
+        gboolean eq2 = icon2 ? g_str_has_prefix (icon2, icons[i]) : FALSE;
         if (eq1 && eq2)
             return 0;
         else if (eq1)
